@@ -1,7 +1,7 @@
 ##############################################################################
 #
-#                        Crossbar.io FX
-#     Copyright (C) Crossbar.io Technologies GmbH. All rights reserved.
+#                        Crossbar.io Database
+#     Copyright (c) Crossbar.io Technologies GmbH. Licensed under MIT.
 #
 ##############################################################################
 
@@ -13,9 +13,5 @@ from cfxdb.log.mworker_log import MWorkerLog
 @table('5ceaa500-4832-451c-adf4-4fc4968cece0', build=MWorkerLog.build, cast=MWorkerLog.cast)
 class MWorkerLogs(MapTimestampUuidStringFlatBuffers):
     """
-    Persisted managed node worker heartbeat log records.
-
-    * Table type :class:`zlmdb.MapTimestampUuidStringFlatBuffers`
-    * Key ``(timestamp, node_id, worker_id)`` type :class:`typing.Tuple[int, uuid.UUID, uuid.UUID]`
-    * Record type :class:`cfxdb.log.mworker_log.MWorkerLog`
+    Managed node worker heartbeat log records (``(timestamp, node_id, worker_id) -> worker_log``).
     """
