@@ -1,7 +1,7 @@
 ##############################################################################
 #
-#                        Crossbar.io FX
-#     Copyright (C) Crossbar.io Technologies GmbH. All rights reserved.
+#                        Crossbar.io Database
+#     Copyright (c) Crossbar.io Technologies GmbH. Licensed under MIT.
 #
 ##############################################################################
 
@@ -233,7 +233,7 @@ class UserFbs(object):
             for o in reversed(_tags):
                 builder.PrependUOffsetTRelative(o)
 
-            tags = builder.EndVector(len(_tags))
+            tags = builder.EndVector()
 
         # now start and build a new object ..
         UserGenFbs.UserStart(builder)
